/*
 * Name: main.c
 * Description: main() function and parameter parsing.
 * Author: Christian Starkjohann <cs@hal.kph.tuwien.ac.at>
 * Date: 1996-11-14
 * Copyright: GNU-GPL
 * Tabsize: 4
 */

#include <libc.h>
#include "my_defines.h"

extern void	nfs_program_2();
extern void	mount_and_dispatch(char *dir, void (*proc)(), void *root_fh);

struct partition {
	unsigned char boot_ind;		/* 0x80 - active */
	unsigned char head;			/* starting head */
	unsigned char sector;		/* starting sector */
	unsigned char cyl;			/* starting cylinder */
	unsigned char sys_ind;		/* What partition type */
	unsigned char end_head;		/* end head */
	unsigned char end_sector;	/* end sector */
	unsigned char end_cyl;		/* end cylinder */
	unsigned int start_sect;	/* starting sector counting from 0 */
	unsigned int nr_sects;		/* nr of sectors in partition */
};

#define PART_TABLE_OFFSET  0x1be	/* offset in the master boot sector */
#define PART_MAX    4 				/* biggest primary partition number */


/* ------------------------------------------------------------------------- */

int				my_fd;
int				wr_enable = 0;
unsigned long	part_offset = 0;
unsigned long	part_size = 0xffffffffUL;
int				debug_mode = 0;

char	root_fh[32] = {0};

/* ------------------------------------------------------------------------- */

static void	get_partition_table(int fd, struct partition *dest)
{
char	sector[512];
int		len = sizeof(struct partition) * PART_MAX;

	if(read(fd, sector, sizeof(sector)) != sizeof(sector)){
		perror("reading partition table");
		exit(1);
	}
	memcpy(dest, sector + PART_TABLE_OFFSET, len);
}

/* ------------------------------------------------------------------------- */

static void	print_partitions(int fd)
{
struct partition	part_table[PART_MAX];
int					i;

	get_partition_table(fd, part_table);
	for(i=0;i<PART_MAX;i++){
		if(part_table[i].start_sect > 0){
			printf("%cpartition %d: type 0x%02x start-sector: 0x%08x\n",
					part_table[i].boot_ind ? '*':' ', i,
					part_table[i].sys_ind, part_table[i].start_sect);
		}
	}
}

/* ------------------------------------------------------------------------- */

static	void set_partition_data(int fd, int part, int sect_size)
{
struct partition	part_table[PART_MAX];

	if(part < 0 || part >= PART_MAX){
		fprintf(stderr, "partition must be >= 0 and < %d (not %d)\n",
															PART_MAX, part);
		exit(1);
	}
	get_partition_table(fd, part_table);
	part_offset = (unsigned long)part_table[part].start_sect * sect_size;
	part_size = (unsigned long)part_table[part].nr_sects * sect_size;
}

/* ------------------------------------------------------------------------- */

static void	print_help(char *name)
{
	fprintf(stderr, "Usage: %s [-i] -m<mount-point> -t<fs-type> [-f<offset> | -p<partition>]\n", name);
	fprintf(stderr, "\t\t[-s<sector-size>] [-w] [-o<options>] [-d<debug-mode>]\n");
	fprintf(stderr, "\t\t[-u <uid-translation-file>] [-g <gid-translation-file>] device\n");
	fprintf(stderr, "-w .... mount filesystem read/write (default: read only)\n");
	fprintf(stderr, "-i .... print partition table of device\n");
}

/* ------------------------------------------------------------------------- */

int	main(int argc, char **argv)
{
char			print_info = 0, read_only = 1, err = 0, *fs_name = NULL;
char			*name = NULL, *mountpoint = NULL, *mount_options = NULL;
int				c, part = -1, sectsize = 512;
long			offset = -1;
char			mountpoint_abs[MAXPATHLEN + 1];
extern char		*optarg;
extern int		optind;

	while ((c = getopt(argc, argv, "o:t:d:t:him:f:p:s:wu:g:")) != EOF){
		switch(c){
		case 'i':
			print_info = 1;
			break;
		case 'm':
			mountpoint = optarg;
			break;
		case 'f':
			offset = strtol(optarg, NULL, 0);
			break;
		case 'p':
			part = atoi(optarg);
			break;
		case 's':
			sectsize = strtol(optarg, NULL, 0);
			break;
		case 'w':
			read_only = 0;
			break;
		case 'h':
			err = 1;
			break;
		case 'o':
			mount_options = optarg;
			break;
		case 'd':
			debug_mode = strtol(optarg, NULL, 0);
			break;
		case 't':
			fs_name = optarg;
			break;
		case 'u':
			load_translationfile(IDBUF_USR, optarg);
			break;
		case 'g':
			load_translationfile(IDBUF_GRP, optarg);
			break;
		default:
			err = 1;
			fprintf(stderr, "unknown option -%c\n", c);
			break;
		}
	}
	if(err){
		print_help(argv[0]);
		exit(1);
	}
	while(optind < argc){
		if(name == NULL)
			name = argv[optind];
		else{
			fprintf(stderr, "trailing superflous parameter ->%s<- ignored\n",
															argv[optind]);
		}
		optind++;
	}
	if(name == NULL){
		fprintf(stderr, "No device specified\n");
		exit(1);
	}
	printf("mounting read%s\n",(wr_enable = !read_only)!=0 ?"/write":" only");
	my_fd = open(name, wr_enable ? O_RDWR : O_RDONLY);
	if(my_fd < 0){
		perror("error opening device");
		exit(1);
	}
	if(print_info){
		print_partitions(my_fd);
		close(my_fd);
		exit(0);
	}
	if(fs_name == NULL){
		fprintf(stderr, "Option -t is mandatory\n");
		exit(1);
	}
	if(mountpoint == NULL){
		fprintf(stderr, "Option -m is mandatory!\n");
		exit(1);
	}
	if(offset != -1 && part != -1){
		fprintf(stderr, "Options -f and -p are exclusive!\n");
		close(my_fd);
		exit(1);
	}
	if(part != -1){
		set_partition_data(my_fd, part, sectsize);
	}else if(offset != -1){
		part_offset = offset;
	}
	missing_init();
	buffer_init();
	dummies_init(name, 512, 512, part_size);
	fops_init();
	if(my_mount(fs_name, wr_enable ? 0 : MNT_RONLY,
									mount_options, (int *)root_fh) != 0){
		close(my_fd);
		exit(1);
	}
	if(*mountpoint == '/'){		/* absolute path given? */
		strcpy(mountpoint_abs, mountpoint);
	}else{
		if(getwd(mountpoint_abs) == NULL){
			mountpoint_abs[0] = 0;
		}else{
			strcat(mountpoint_abs, "/");
		}
		strcat(mountpoint_abs, mountpoint);
	}
	mount_and_dispatch(mountpoint_abs, nfs_program_2, root_fh);
	close(my_fd);
	return 0;
}

/* ------------------------------------------------------------------------- */
